﻿package com.*******;

import org.apache.cordova.CallbackContext;
import org.apache.cordova.CordovaPlugin;
import org.apache.cordova.CordovaActivity;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import com.alivc.player.AccessKey;
import com.alivc.player.AccessKeyCallback;
import com.alivc.player.AliVcMediaPlayer;
import com.alivc.player.MediaPlayer;

import android.content.Context;
import android.content.pm.ApplicationInfo;
import android.content.pm.PackageManager;
import android.graphics.Color;
import android.graphics.Point;
import android.util.Log;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import android.view.View;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;
import android.widget.TextView;
import android.app.Activity; 

public class PlayerPlugin extends CordovaPlugin {

	private static String TAG = "PlayerPlugin";
	
    private CallbackContext callbackContext = null;
    public RelativeLayout relLayout = null;
    public SurfaceView mSurfaceView = null;
    private SurfaceHolder mSurfaceHolder = null;
    //private MainActivity mainActivity;
    private CordovaActivity  mainActivity;
    private TextView textView = null;
    
    public AliVcMediaPlayer mPlayer = null;
    public String url = null;

    @Override
    public boolean execute(String action, JSONArray args, CallbackContext callbackContext) throws JSONException {
        this.callbackContext = callbackContext;
        //判断是否显示播放器界面
        if (action.equals("playVideo")) {
        	 mainActivity = (CordovaActivity) cordova.getActivity();
        	 try {
        		  ApplicationInfo appInfo = mainActivity.getPackageManager().getApplicationInfo(mainActivity.getPackageName(),  
        		            PackageManager.GET_META_DATA);  
        		  final String accessKeyId = appInfo.metaData.getString("accessKeyId"); 
        		  final String accessKeySecret = appInfo.metaData.getString("accessKeySecret"); 
        		  final String businessId = appInfo.metaData.getString("businessId");

                 AliVcMediaPlayer.init(mainActivity.getApplicationContext(), businessId, new AccessKeyCallback() {
                     public AccessKey getAccessToken() {
                         return new AccessKey(accessKeyId, accessKeySecret);
                     }
                 });
                 JSONObject jsonObject = args.getJSONObject(0);
                 url = jsonObject.getString("url");
                 int videoTop = jsonObject.getInt("top");
                 //videoWidth = jsonObject.getInt("width");
                 //videoHeight = jsonObject.getInt("height");
                 //显示或者隐藏
                 playVideo(videoTop);
             } catch (Throwable t){
                 callbackContext.error(t.getMessage().toString());
                 return false;
             }
        	 return true;
        }else if(action.equals("destroy")){
            //销毁
        	destroy();
            return true;
        }
        return false ;
    }

    private boolean playVideo(final int top){
        try {
    		mainActivity.runOnUiThread(new Runnable(){
                @Override
                public void run(){
                	if(mPlayer == null){
                		createView(top);
                	}else{
                		mPlayer.releaseVideoSurface();
                    	mPlayer.stop();
                        mPlayer.destroy();
                        mPlayer = null;
                        startToPlay();
                	}
                }
            });
            return true;
        }catch (Exception e){
            callbackContext.error(getStringValue("errorHint")+":"+e);
            return false;
        }
    }
    
    private void destroy(){
    	if (mainActivity != null){
            mainActivity.runOnUiThread(new Runnable(){
                @Override
                public void run(){
                    if (mPlayer != null){
                    	mPlayer.releaseVideoSurface();
                    	mPlayer.stop();
                        mPlayer.destroy();
                        mPlayer = null;
                    }
                    if (mSurfaceView != null){
                        mSurfaceView.setVisibility(View.GONE);
                        mSurfaceView = null ;
                    }
                    if (relLayout != null){
                        relLayout.setVisibility(View.GONE);
                        relLayout = null ;
                    }
                }
            });
        }
    }

    private void createView(int top){
        //线性布局
    	LinearLayout linearLayout = new LinearLayout(mainActivity);
    	relLayout = new RelativeLayout(mainActivity);
    	LinearLayout.LayoutParams wrapParam = new LinearLayout.LayoutParams(LinearLayout.LayoutParams.MATCH_PARENT, LinearLayout.LayoutParams.MATCH_PARENT);
    	wrapParam.setMargins(0, top, 0, 0);
    	linearLayout.removeAllViews();
    	linearLayout.addView(relLayout, wrapParam);
    	
        RelativeLayout.LayoutParams playParam = new RelativeLayout.LayoutParams(RelativeLayout.LayoutParams.MATCH_PARENT, RelativeLayout.LayoutParams.WRAP_CONTENT);
        //设置放在父控件中间
        playParam.addRule(RelativeLayout.CENTER_IN_PARENT, RelativeLayout.TRUE);
        //创建一个videoView==============用surfaceview代替
        mSurfaceView = new SurfaceView(mainActivity);
        mSurfaceView.setZOrderOnTop(false);
        // 为避免重复添加,事先remove子view
        relLayout.removeAllViews();
        relLayout.addView(mSurfaceView, playParam);

        //显示正在加载
        textView = new TextView(mainActivity);
        textView.setTextSize(20);
        textView.setTextColor(Color.WHITE);
        RelativeLayout.LayoutParams msgParam = new RelativeLayout.LayoutParams(RelativeLayout.LayoutParams.WRAP_CONTENT,RelativeLayout.LayoutParams.WRAP_CONTENT);
        //设置放在父控件中间
        msgParam.addRule(RelativeLayout.CENTER_IN_PARENT, RelativeLayout.TRUE);
        relLayout.addView(textView, msgParam);
        
        Point deviceSize = new Point();
        mainActivity.getWindowManager().getDefaultDisplay().getSize(deviceSize);
        mainActivity.addContentView(linearLayout, new LinearLayout.LayoutParams(LinearLayout.LayoutParams.MATCH_PARENT, deviceSize.x*3/4));

        mSurfaceHolder = mSurfaceView.getHolder();
        mSurfaceHolder.addCallback(mSurfaceHolderCB);
    }

    private SurfaceHolder.Callback mSurfaceHolderCB = new SurfaceHolder.Callback() {
        @SuppressWarnings("deprecation")
        public void surfaceCreated(SurfaceHolder holder) {
            holder.setType(SurfaceHolder.SURFACE_TYPE_GPU);
            holder.setKeepScreenOn(true);
            // 重点:
            if (mPlayer != null) {
                // 对于从后台切换到前台,需要重设surface;部分手机锁屏也会做前后台切换的处理
                mPlayer.setVideoSurface(mSurfaceView.getHolder().getSurface());
            }else {
                // 创建并启动播放器
                startToPlay();
            }
        }

        public void surfaceChanged(SurfaceHolder holder, int format, int w, int h) {
            if (mPlayer != null)
                mPlayer.setSurfaceChanged();
        }

        public void surfaceDestroyed(SurfaceHolder holder) {
            if (mPlayer != null) {
                mPlayer.releaseVideoSurface();
            }
        }
    };

    private boolean startToPlay(){
    	try {
            if (mPlayer == null){
                // 初始化播放器
                mPlayer = new AliVcMediaPlayer(mainActivity, mSurfaceView);
                mPlayer.setPreparedListener(new VideoPreparedListener());//播放器就绪事件
                mPlayer.setErrorListener(new VideoErrorListener());	 	//异常错误事件
                mPlayer.setInfoListener(new VideoInfolistener());		//信息状态监听事件 
                mPlayer.setCompletedListener(new VideoCompletelistener());//播放结束事件
                mPlayer.setVideoSizeChangeListener(new VideoSizeChangelistener());//画面大小变化事件
                mPlayer.setBufferingUpdateListener(new VideoBufferUpdatelistener());//缓冲信息更新事件
                mPlayer.setStopedListener(new VideoStoppedListener());//播放停止事件
                mPlayer.setDefaultDecoder(0);		// 如果同时支持软解和硬解是有用
            }
            textView.setVisibility(View.VISIBLE);
        	textView.setText(getStringValue("willPlay"));
            //设置播放的url
            mPlayer.prepareAndPlay(url);
        }catch (Exception e){
            callbackContext.error(e.getMessage().toString());
        }
        return true;
    }

    /**
     * 准备完成监听器:调度更新进度
     */
    private class VideoPreparedListener implements MediaPlayer.MediaPlayerPreparedListener {
        @Override
        public void onPrepared() {
            textView.setVisibility(View.GONE);
        }
    }

    /**
     * 错误处理监听器
     */
    private class VideoErrorListener implements MediaPlayer.MediaPlayerErrorListener {

        public void onError(int what, int extra) {
            int errCode = 0;
            if (mPlayer == null) {
                return;
            }
            errCode = mPlayer.getErrorCode();
            switch (errCode) {
                case MediaPlayer.ALIVC_ERR_LOADING_TIMEOUT:
                    //把结果返回给js
                	textView.setText(getStringValue("ALIVC_ERR_LOADING_TIMEOUT"));
                    callbackContext.error(getStringValue("ALIVC_ERR_LOADING_TIMEOUT"));
                    break;
                case MediaPlayer.ALIVC_ERR_NO_INPUTFILE:
                	textView.setText(getStringValue("ALIVC_ERR_NO_INPUTFILE"));
                    callbackContext.error(getStringValue("ALIVC_ERR_NO_INPUTFILE"));
                    break;
                case MediaPlayer.ALIVC_ERR_NO_VIEW:
                	textView.setText(getStringValue("ALIVC_ERR_NO_VIEW"));
                    callbackContext.error(getStringValue("ALIVC_ERR_NO_VIEW"));
                    break;
                case MediaPlayer.ALIVC_ERR_INVALID_INPUTFILE:
                	textView.setText(getStringValue("ALIVC_ERR_INVALID_INPUTFILE"));
                    callbackContext.error(getStringValue("ALIVC_ERR_INVALID_INPUTFILE"));
                    break;
                case MediaPlayer.ALIVC_ERR_NO_SUPPORT_CODEC:
                	textView.setText(getStringValue("ALIVC_ERR_NO_SUPPORT_CODEC"));
                    callbackContext.error(getStringValue("ALIVC_ERR_NO_SUPPORT_CODEC"));
                    break;
                case MediaPlayer.ALIVC_ERR_FUNCTION_DENIED:
                	textView.setText(getStringValue("ALIVC_ERR_FUNCTION_DENIED"));
                    callbackContext.error(getStringValue("ALIVC_ERR_FUNCTION_DENIED"));
                    break;
                case MediaPlayer.ALIVC_ERR_UNKNOWN:
                	textView.setText(getStringValue("ALIVC_ERR_UNKNOWN"));
                    callbackContext.error(getStringValue("ALIVC_ERR_UNKNOWN"));
                    break;
                case MediaPlayer.ALIVC_ERR_NO_NETWORK:
                	textView.setText(getStringValue("ALIVC_ERR_NO_NETWORK"));
                    callbackContext.error(getStringValue("ALIVC_ERR_NO_NETWORK"));
                    break;
                case MediaPlayer.ALIVC_ERR_ILLEGALSTATUS:
                	textView.setText(getStringValue(getStringValue("ALIVC_ERR_ILLEGALSTATUS")));
                    //非法状态
                    callbackContext.error(getStringValue("ALIVC_ERR_ILLEGALSTATUS"));
                    break;
                case MediaPlayer.ALIVC_ERR_NOTAUTH:
                	textView.setText(getStringValue("ALIVC_ERR_NOTAUTH"));
                    callbackContext.error(getStringValue(getStringValue("ALIVC_ERR_NOTAUTH")));
                    break;
                case MediaPlayer.ALIVC_ERR_READD:
                	textView.setText(getStringValue("ALIVC_ERR_READD"));
                    callbackContext.error(getStringValue("ALIVC_ERR_READD"));
                    break;
                default:
                    break;
            }
        }
    }

    /**
     * 信息通知监听器:重点是缓存开始/结束
     */
    private class VideoInfolistener implements MediaPlayer.MediaPlayerInfoListener {

        public void onInfo(int what, int extra) {
            switch (what) {
                case MediaPlayer.MEDIA_INFO_UNKNOW:
                    break;
                case MediaPlayer.MEDIA_INFO_BUFFERING_START:
                    break;
                case MediaPlayer.MEDIA_INFO_BUFFERING_END:
                	textView.setVisibility(View.GONE);
                    break;
                case MediaPlayer.MEDIA_INFO_TRACKING_LAGGING:
                    break;
                case MediaPlayer.MEDIA_INFO_NETWORK_ERROR:
                	textView.setText(getStringValue("MEDIA_INFO_NETWORK_ERROR"));
                    break;
                case MediaPlayer.MEDIA_INFO_VIDEO_RENDERING_START:
                    break;
            }
        }
    }

    /**
     * 视频播完监听器
     */
    private class VideoCompletelistener implements MediaPlayer.MediaPlayerCompletedListener {

        public void onCompleted() {
        }
    }

    /**
     * 视频大小变化监听器
     */
    private class VideoSizeChangelistener implements MediaPlayer.MediaPlayerVideoSizeChangeListener {

        public void onVideoSizeChange(int width, int height) {
        }
    }

    /**
     * 视频缓存变化监听器: percent 为 0~100之间的数字】
     */
    private class VideoBufferUpdatelistener implements MediaPlayer.MediaPlayerBufferingUpdateListener {

        public void onBufferingUpdateListener(int percent) {
            textView.setVisibility(View.VISIBLE);
        }
    }

    private class VideoStoppedListener implements MediaPlayer.MediaPlayerStopedListener {
        @Override
        public void onStopped(){
        }
    }
    
    private String getStringValue(String key){
    	//return (String) mainActivity.getResources().getString(R.string.app_name); 
    	//初始化
    	try {
    		int id = cordova.getActivity().getResources().getIdentifier(key, "string", cordova.getActivity().getPackageName());
            return cordova.getActivity().getString(id) ;
		} catch (Exception e) {
			return "" ;
		}
    }
}
